@@ -1,18 +1,17 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# -*- test-case-name: twisted.conch.test.test_telnet -*-
+# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-"""Telnet protocol implementation.
+"""
+Telnet protocol implementation.
 
-API Stability: Unstable
-
-@author: U{Jp Calderone<mailto:exarkun@twistedmatrix.com>}
+@author: Jp Calderone
 """
 
 import struct
 
 from zope.interface import implements
 
-from twisted.application import internet
 from twisted.internet import protocol, interfaces as iinternet, defer
 from twisted.python import log
 
@@ -154,15 +153,15 @@ LINEMODE_SUSP = chr(237)
 LINEMODE_ABORT = chr(238)
 
 class ITelnetProtocol(iinternet.IProtocol):
-    def unhandledCommand(self, command, argument):
+    def unhandledCommand(command, argument):
         """A command was received but not understood.
         """
 
-    def unhandledSubnegotiation(self, bytes):
+    def unhandledSubnegotiation(bytes):
         """A subnegotiation command was received but not understood.
         """
 
-    def enableLocal(self, option):
+    def enableLocal(option):
         """Enable the given option locally.
 
         This should enable the given option on this side of the
@@ -171,83 +170,83 @@ class ITelnetProtocol(iinternet.IProtocol):
         will be notified.
         """
 
-    def enableRemote(self, option):
+    def enableRemote(option):
         """Indicate whether the peer should be allowed to enable this option.
 
         Returns True if the peer should be allowed to enable this option,
         False otherwise.
         """
 
-    def disableLocal(self, option):
+    def disableLocal(option):
         """Disable the given option locally.
 
         Unlike enableLocal, this method cannot fail.  The option must be
         disabled.
         """
 
-    def disableRemote(self, option):
+    def disableRemote(option):
         """Indicate that the peer has disabled this option.
         """
 
 class ITelnetTransport(iinternet.ITransport):
-    def do(self, option):
+    def do(option):
         """Indicate a desire for the peer to begin performing the given option.
 
         Returns a Deferred that fires with True when the peer begins performing
         the option, or False when the peer refuses to perform it.  If the peer
         is already performing the given option, the Deferred will fail with
-        C{AlreadyEnabled}.  If a negotiation regarding this option is already
-        in progress, the Deferred will fail with C{AlreadyNegotiating}.
+        L{AlreadyEnabled}.  If a negotiation regarding this option is already
+        in progress, the Deferred will fail with L{AlreadyNegotiating}.
 
         Note: It is currently possible that this Deferred will never fire,
         if the peer never responds, or if the peer believes the option to
         already be enabled.
         """
 
-    def dont(self, option):
+    def dont(option):
         """Indicate a desire for the peer to cease performing the given option.
 
         Returns a Deferred that fires with True when the peer ceases performing
         the option.  If the peer is not performing the given option, the
-        Deferred will fail with C{AlreadyDisabled}.  If negotiation regarding
+        Deferred will fail with L{AlreadyDisabled}.  If negotiation regarding
         this option is already in progress, the Deferred will fail with
-        C{AlreadyNegotiating}.
+        L{AlreadyNegotiating}.
 
         Note: It is currently possible that this Deferred will never fire,
         if the peer never responds, or if the peer believes the option to
         already be disabled.
         """
 
-    def will(self, option):
+    def will(option):
         """Indicate our willingness to begin performing this option locally.
 
         Returns a Deferred that fires with True when the peer agrees to allow
         us to begin performing this option, or False if the peer refuses to
         allow us to begin performing it.  If the option is already enabled
-        locally, the Deferred will fail with C{AlreadyEnabled}.  If negotiation
+        locally, the Deferred will fail with L{AlreadyEnabled}.  If negotiation
         regarding this option is already in progress, the Deferred will fail with
-        C{AlreadyNegotiating}.
+        L{AlreadyNegotiating}.
 
         Note: It is currently possible that this Deferred will never fire,
         if the peer never responds, or if the peer believes the option to
         already be enabled.
         """
 
-    def wont(self, option):
+    def wont(option):
         """Indicate that we will stop performing the given option.
 
         Returns a Deferred that fires with True when the peer acknowledges
         we have stopped performing this option.  If the option is already
-        disabled locally, the Deferred will fail with C{AlreadyDisabled}.
+        disabled locally, the Deferred will fail with L{AlreadyDisabled}.
         If negotiation regarding this option is already in progress,
-        the Deferred will fail with C{AlreadyNegotiation}.
+        the Deferred will fail with L{AlreadyNegotiating}.
 
         Note: It is currently possible that this Deferred will never fire,
         if the peer never responds, or if the peer believes the option to
         already be disabled.
         """
 
-    def requestNegotiation(self, about, bytes):
+    def requestNegotiation(about, bytes):
         """Send a subnegotiation request.
 
         @param about: A byte indicating the feature being negotiated.
@@ -433,7 +432,8 @@ class Telnet(protocol.Protocol):
         self._write(IAC + SB + bytes + IAC + SE)
 
     def dataReceived(self, data):
-        # Most grossly inefficient implementation ever
+        appDataBuffer = []
+
         for b in data:
             if self.state == 'data':
                 if b == IAC:
@@ -441,16 +441,19 @@ class Telnet(protocol.Protocol):
                 elif b == '\r':
                     self.state = 'newline'
                 else:
-                    self.applicationDataReceived(b)
+                    appDataBuffer.append(b)
             elif self.state == 'escaped':
                 if b == IAC:
-                    self.applicationDataReceived(b)
+                    appDataBuffer.append(b)
                     self.state = 'data'
                 elif b == SB:
                     self.state = 'subnegotiation'
                     self.commands = []
                 elif b in (NOP, DM, BRK, IP, AO, AYT, EC, EL, GA):
                     self.state = 'data'
+                    if appDataBuffer:
+                        self.applicationDataReceived(''.join(appDataBuffer))
+                        del appDataBuffer[:]
                     self.commandReceived(b, None)
                 elif b in (WILL, WONT, DO, DONT):
                     self.state = 'command'
@@ -461,15 +464,31 @@ class Telnet(protocol.Protocol):
                 self.state = 'data'
                 command = self.command
                 del self.command
+                if appDataBuffer:
+                    self.applicationDataReceived(''.join(appDataBuffer))
+                    del appDataBuffer[:]
                 self.commandReceived(command, b)
             elif self.state == 'newline':
-                if b == '\n':
-                    self.applicationDataReceived('\n')
-                elif b == '\0':
-                    self.applicationDataReceived('\r')
-                else:
-                    self.applicationDataReceived('\r' + b)
                 self.state = 'data'
+                if b == '\n':
+                    appDataBuffer.append('\n')
+                elif b == '\0':
+                    appDataBuffer.append('\r')
+                elif b == IAC:
+                    # IAC isn't really allowed after \r, according to the
+                    # RFC, but handling it this way is less surprising than
+                    # delivering the IAC to the app as application data. 
+                    # The purpose of the restriction is to allow terminals
+                    # to unambiguously interpret the behavior of the CR
+                    # after reading only one more byte.  CR LF is supposed
+                    # to mean one thing (cursor to next line, first column),
+                    # CR NUL another (cursor to first column).  Absent the
+                    # NUL, it still makes sense to interpret this as CR and
+                    # then apply all the usual interpretation to the IAC.
+                    appDataBuffer.append('\r')
+                    self.state = 'escaped'
+                else:
+                    appDataBuffer.append('\r' + b)
             elif self.state == 'subnegotiation':
                 if b == IAC:
                     self.state = 'subnegotiation-escaped'
@@ -480,6 +499,9 @@ class Telnet(protocol.Protocol):
                     self.state = 'data'
                     commands = self.commands
                     del self.commands
+                    if appDataBuffer:
+                        self.applicationDataReceived(''.join(appDataBuffer))
+                        del appDataBuffer[:]
                     self.negotiate(commands)
                 else:
                     self.state = 'subnegotiation'
@@ -487,6 +509,10 @@ class Telnet(protocol.Protocol):
             else:
                 raise ValueError("How'd you do this?")
 
+        if appDataBuffer:
+            self.applicationDataReceived(''.join(appDataBuffer))
+
+
     def connectionLost(self, reason):
         for state in self.options.values():
             if state.us.onResult is not None:
@@ -662,16 +688,49 @@ class Telnet(protocol.Protocol):
                ('yes', False): dont_yes_false, ('yes', True): dont_yes_true}
 
     def enableLocal(self, option):
-        return self.protocol.enableLocal(option)
+        """
+        Reject all attempts to enable options.
+        """
+        return False
+
 
     def enableRemote(self, option):
-        return self.protocol.enableRemote(option)
+        """
+        Reject all attempts to enable options.
+        """
+        return False
+
 
     def disableLocal(self, option):
-        return self.protocol.disableLocal(option)
+        """
+        Signal a programming error by raising an exception.
+
+        L{enableLocal} must return true for the given value of C{option} in
+        order for this method to be called.  If a subclass of L{Telnet}
+        overrides enableLocal to allow certain options to be enabled, it must
+        also override disableLocal to disable those options.
+
+        @raise NotImplementedError: Always raised.
+        """
+        raise NotImplementedError(
+            "Don't know how to disable local telnet option %r" % (option,))
+
 
     def disableRemote(self, option):
-        return self.protocol.disableRemote(option)
+        """
+        Signal a programming error by raising an exception.
+
+        L{enableRemote} must return true for the given value of C{option} in
+        order for this method to be called.  If a subclass of L{Telnet}
+        overrides enableRemote to allow certain options to be enabled, it must
+        also override disableRemote tto disable those options.
+
+        @raise NotImplementedError: Always raised.
+        """
+        raise NotImplementedError(
+            "Don't know how to disable remote telnet option %r" % (option,))
+
+
 
 class ProtocolTransportMixin:
     def write(self, bytes):
@@ -696,7 +755,7 @@ class TelnetTransport(Telnet, ProtocolTransportMixin):
     established and after it is lost.
 
     @ivar protocolFactory: A callable which returns protocol instances
-    which provide ITelnetProtocol.  This will be invoked when a
+    which provide L{ITelnetProtocol}.  This will be invoked when a
     connection is established.  It is passed *protocolArgs and
     **protocolKwArgs.
 
@@ -880,7 +939,7 @@ class AuthenticatingTelnetProtocol(StatefulTelnetProtocol):
 
     Username and password prompts are given (the password is obscured).  When the
     information is collected, it is passed to a portal and an avatar implementing
-    ITelnetProtocol is requested.  If an avatar is returned, it connected to this
+    L{ITelnetProtocol} is requested.  If an avatar is returned, it connected to this
     protocol's transport, and this protocol's transport is connected to it.
     Otherwise, the user is re-prompted for credentials.
     """
